@@ -81,7 +81,7 @@ static void HandleShapeEvent(const XShapeEvent *event);
 #endif
 
 /** Wait for an event and process it. */
-void WaitForEvent(XEvent *event)
+char WaitForEvent(XEvent *event)
 {
 
    struct timeval timeout;
@@ -115,6 +115,9 @@ void WaitForEvent(XEvent *event)
          if(select(fd + 1, &fds, NULL, NULL, &timeout) <= 0) {
             Signal();
          }
+         if(JUNLIKELY(shouldExit)) {
+            return 0;
+         }
       }
 
       Signal();
@@ -220,6 +223,8 @@ void WaitForEvent(XEvent *event)
 
    } while(handled && !shouldExit);
 
+   return !handled;
+
 }
 
 /** Wake up components that need to run at certain times. */
@@ -286,6 +291,7 @@ void ProcessEvent(XEvent *event)
 void DiscardMotionEvents(XEvent *event, Window w)
 {
    XEvent temp;
+   JXSync(display, False);
    while(JXCheckTypedEvent(display, MotionNotify, &temp)) {
       UpdateTime(&temp);
       SetMousePosition(temp.xmotion.x_root, temp.xmotion.y_root,
@@ -299,6 +305,7 @@ void DiscardMotionEvents(XEvent *event, Window w)
 /** Discard key events for the specified window. */
 void DiscardKeyEvents(XEvent *event, Window w)
 {
+   JXSync(display, False);
    while(JXCheckTypedWindowEvent(display, w, KeyPress, event)) {
       UpdateTime(event);
    }
@@ -343,7 +350,7 @@ void HandleButtonEvent(const XButtonEvent *event)
             RaiseClient(np);
             if(mask == Mod1Mask) {
                GetBorderSize(&np->state, &north, &south, &east, &west);
-               MoveClient(np, event->x + west, event->y + north, 0);
+               MoveClient(np, event->x + west, event->y + north);
             }
             break;
          case Button3:
@@ -515,7 +522,7 @@ void HandleConfigureRequest(const XConfigureRequestEvent *event)
       char resized = 0;
       int deltax, deltay;
 
-      GetGravityDelta(np, &deltax, &deltay);
+      GetGravityDelta(np, np->gravity, &deltax, &deltay);
       if((event->value_mask & CWWidth) && (event->width != np->width)) {
          switch(np->gravity) {
          case EastGravity:
@@ -695,6 +702,9 @@ char HandlePropertyNotify(const XPropertyEvent *event)
          break;
       case XA_WM_NORMAL_HINTS:
          ReadWMNormalHints(np);
+         if(ConstrainSize(np)) {
+            ResetBorder(np);
+         }
          changed = 1;
          break;
       case XA_WM_HINTS:
@@ -871,31 +881,70 @@ void HandleNetMoveResize(const XClientMessageEvent *event, ClientNode *np)
 {
 
    long flags;
-   long x, y;
-   long width, height;
+   int gravity;
    int deltax, deltay;
 
    Assert(event);
    Assert(np);
 
    flags = event->data.l[0] >> 8;
-
-   x = np->x;
-   y = np->y;
-   width = np->width;
-   height = np->height;
-
-   if(flags & (1 << 0)) {
-      x = event->data.l[1];
-   }
-   if(flags & (1 << 1)) {
-      y = event->data.l[2];
+   gravity = event->data.l[0] & 0xFF;
+   if(gravity == 0) {
+      gravity = np->gravity;
    }
+   GetGravityDelta(np, gravity, &deltax, &deltay);
+
    if(flags & (1 << 2)) {
-      width = event->data.l[3];
+      const long width = event->data.l[3];
+      switch(gravity) {
+      case EastGravity:
+      case NorthEastGravity:
+      case SouthEastGravity:
+         /* Right side should not move. */
+         np->x -= width - np->width;
+         break;
+      case WestGravity:
+      case NorthWestGravity:
+      case SouthWestGravity:
+         /* Left side should not move. */
+         break;
+      case CenterGravity:
+         /* Center of the window should not move. */
+         np->x -= (width - np->width) / 2;
+         break;
+      default:
+         break;
+      }
+      np->width = width;
    }
    if(flags & (1 << 3)) {
-      height = event->data.l[4];
+      const long height = event->data.l[4];
+      switch(gravity) {
+      case NorthGravity:
+      case NorthEastGravity:
+      case NorthWestGravity:
+         /* Top should not move. */
+         break;
+      case SouthGravity:
+      case SouthEastGravity:
+      case SouthWestGravity:
+         /* Bottom should not move. */
+         np->y -= height - np->height;
+         break;
+      case CenterGravity:
+         /* Center of the window should not move. */
+         np->y -= (height - np->height) / 2;
+         break;
+      default:
+         break;
+      }
+      np->height = height;
+   }
+   if(flags & (1 << 0)) {
+      np->x = event->data.l[1] - deltax;
+   }
+   if(flags & (1 << 1)) {
+      np->y = event->data.l[2] - deltay;
    }
 
    /* Don't let maximized clients be moved or resized. */
@@ -906,17 +955,7 @@ void HandleNetMoveResize(const XClientMessageEvent *event, ClientNode *np)
       MaximizeClient(np, 0, 0);
    }
 
-   GetGravityDelta(np, &deltax, &deltay);
-   x -= deltax;
-   y -= deltay;
-
-   np->x = x;
-   np->y = y;
-   np->width = width;
-   np->height = height;
    ConstrainSize(np);
-
-   WriteState(np);
    ResetBorder(np);
    SendConfigureEvent(np);
    UpdatePager();
@@ -1104,6 +1143,14 @@ void HandleNetWMState(const XClientMessageEvent *event, ClientNode *np)
       Debug("bad _NET_WM_STATE action: %ld", event->data.l[0]);
       break;
    }
+
+   /* Update _NET_WM_STATE if needed.
+    * The state update is handled elsewhere for the other actions.
+    */
+   if(actionNolist | actionNopager | actionAbove | actionBelow) {
+      WriteState(np);
+   }
+
 }
 
 /** Handle a _NET_REQUEST_FRAME_EXTENTS request. */
@@ -1278,7 +1325,7 @@ void DispatchBorderButtonEvent(const XButtonEvent *event,
    /* Middle click starts a move unless it's over the maximize button. */
    action = GetBorderActionType(np, event->x, event->y);
    if(event->button == Button2 && action != BA_MAXIMIZE) {
-      MoveClient(np, event->x, event->y, (event->state & Mod1Mask) ? 0 : 1);
+      MoveClient(np, event->x, event->y);
       return;
    }
 
@@ -1312,8 +1359,7 @@ void DispatchBorderButtonEvent(const XButtonEvent *event,
                MaximizeClientDefault(np);
                doubleClickActive = 0;
             } else {
-               if(MoveClient(np, event->x, event->y,
-                             (event->state & Mod1Mask) ? 0 : 1)) {
+               if(MoveClient(np, event->x, event->y)) {
                   doubleClickActive = 0;
                } else {
                   doubleClickActive = 1;
