@@ -298,11 +298,12 @@ void ReadClientInfo(ClientNode *np, char alreadyMapped)
    /* Read WM protocols. */
    ReadWMProtocols(np->window, &np->state);
 
-   /* Set the client to the same layer as its owner. */
+   /* Make sure this client is on at least as high of a layer
+    * as its owner. */
    if(np->owner != None) {
       pp = FindClientByWindow(np->owner);
       if(pp) {
-         np->state.layer = pp->state.layer;
+         np->state.layer = Max(pp->state.layer, np->state.layer);
       }
    }
 
@@ -846,18 +847,6 @@ void ReadWMNormalHints(ClientNode *np)
       np->aspect.miny = hints.min_aspect.y;
       np->aspect.maxx = hints.max_aspect.x;
       np->aspect.maxy = hints.max_aspect.y;
-      if(np->aspect.minx < 1) {
-         np->aspect.minx = 1;
-      }
-      if(np->aspect.miny < 1) {
-         np->aspect.miny = 1;
-      }
-      if(np->aspect.maxx < 1) {
-         np->aspect.maxx = 1;
-      }
-      if(np->aspect.maxy < 1) {
-         np->aspect.maxy = 1;
-      }
    }
 
    if(np->sizeFlags & PWinGravity) {
