@@ -16,6 +16,7 @@
 #include "main.h"
 #include "settings.h"
 #include "clientlist.h"
+#include "misc.h"
 
 typedef struct Strut {
    ClientNode *client;
@@ -653,14 +654,20 @@ void PlaceClient(ClientNode *np, char alreadyMapped)
 }
 
 /** Constrain the size of the client. */
-void ConstrainSize(ClientNode *np)
+char ConstrainSize(ClientNode *np)
 {
 
    BoundingBox box;
    const ScreenType *sp;
    int north, south, east, west;
+   const int oldWidth = np->width;
+   const int oldHeight = np->height;
 
-   Assert(np);
+   /* First we make sure the window isn't larger than the program allows.
+    * We do this here to avoid moving the window below.
+    */
+   np->width = Min(np->width, np->maxWidth);
+   np->height = Min(np->height, np->maxHeight);
 
    /* Constrain the width if necessary. */
    sp = GetCurrentScreen(np->x, np->y);
@@ -695,21 +702,25 @@ void ConstrainSize(ClientNode *np)
       np->height = box.height - (box.height % np->yinc);
    }
 
+   /* If the program has a minimum constraint, we apply that here.
+    * Note that this could cause the window to overlap something. */
+   np->width = Max(np->width, np->minWidth);
+   np->height = Max(np->height, np->minHeight);
+
    /* Fix the aspect ratio. */
    if(np->sizeFlags & PAspect) {
-
-      /* Fixed point with a 16-bit fraction. */
-      const int ratio = (np->width << 16) / np->height;
-      int minr, maxr;
-      minr = (np->aspect.minx << 16) / np->aspect.miny;
-      if(ratio < minr) {
-         np->height = (np->width << 16) / minr;
+      if(np->width * np->aspect.miny < np->height * np->aspect.minx) {
+         np->height = (np->width * np->aspect.miny) / np->aspect.minx;
       }
-      maxr = (np->aspect.maxx << 16) / np->aspect.maxy;
-      if(ratio > maxr) {
-         np->width = (np->height * maxr) >> 16;
+      if(np->width * np->aspect.maxy > np->height * np->aspect.maxx) {
+         np->width = (np->height * np->aspect.maxx) / np->aspect.maxy;
       }
+   }
 
+   if(np->width != oldWidth || np->height != oldHeight) {
+      return 1;
+   } else {
+      return 0;
    }
 
 }
@@ -778,21 +789,12 @@ void PlaceMaximizedClient(ClientNode *np, char horiz, char vert)
    }
 
    if(np->sizeFlags & PAspect) {
-
-      /* Fixed point with a 16-bit fraction. */
-      const int ratio = (box.width << 16) / box.height;
-      int minr, maxr;
-
-      minr = (np->aspect.minx << 16) / np->aspect.miny;
-      if(ratio < minr) {
-         box.height = (box.width << 16) / minr;
+      if(box.width * np->aspect.miny < box.height * np->aspect.minx) {
+         box.height = (box.width * np->aspect.miny) / np->aspect.minx;
       }
-
-      maxr = (np->aspect.maxx << 16) / np->aspect.maxy;
-      if(ratio > maxr) {
-         box.width = (box.height * maxr) >> 16;
+      if(box.width * np->aspect.maxy > box.height * np->aspect.maxx) {
+         box.width = (box.height * np->aspect.maxx) / np->aspect.maxy;
       }
-
    }
 
    /* If maximizing horizontally, update width. */
@@ -818,18 +820,12 @@ void PlaceMaximizedClient(ClientNode *np, char horiz, char vert)
 }
 
 /** Determine which way to move the client for the border. */
-void GetGravityDelta(const ClientNode *np, int *x, int  *y)
+void GetGravityDelta(const ClientNode *np, int gravity, int *x, int  *y)
 {
 
    int north, south, east, west;
-
-   Assert(np);
-   Assert(x);
-   Assert(y);
-
    GetBorderSize(&np->state, &north, &south, &east, &west);
-
-   switch(np->gravity) {
+   switch(gravity) {
    case NorthWestGravity:
       *y = -north;
       *x = -west;
@@ -879,13 +875,7 @@ void GravitateClient(ClientNode *np, char negate)
 {
 
    int deltax, deltay;
-
-   Assert(np);
-
-   deltax = 0;
-   deltay = 0;
-   GetGravityDelta(np, &deltax, &deltay);
-
+   GetGravityDelta(np, np->gravity, &deltax, &deltay);
    if(negate) {
       np->x += deltax;
       np->y += deltay;
